"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/camelcase */
const crypto_1 = require("@iov/crypto");
const encoding_1 = require("@iov/encoding");
const utils_1 = require("@iov/utils");
const readonly_date_1 = require("readonly-date");
const address_1 = require("./address");
const encoding_2 = require("./encoding");
const logs_1 = require("./logs");
const pen_1 = require("./pen");
const pubkey_1 = require("./pubkey");
const restclient_1 = require("./restclient");
const signingcosmwasmclient_1 = require("./signingcosmwasmclient");
const cosmoshub_json_1 = __importDefault(require("./testdata/cosmoshub.json"));
const testutils_spec_1 = require("./testutils.spec");
const types_1 = require("./types");
const { fromAscii, fromBase64, fromHex, toAscii, toBase64, toHex } = encoding_1.Encoding;
const emptyAddress = "cosmos1ltkhnmdcqemmd2tkhnx7qx66tq7e0wykw2j85k";
function makeSignedTx(firstMsg, fee, memo, firstSignature) {
    return {
        msg: [firstMsg],
        fee: fee,
        memo: memo,
        signatures: [firstSignature],
    };
}
async function uploadCustomContract(client, pen, wasmCode) {
    const memo = "My first contract on chain";
    const theMsg = {
        type: "wasm/MsgStoreCode",
        value: {
            sender: testutils_spec_1.faucet.address,
            wasm_byte_code: toBase64(wasmCode),
            source: "https://github.com/confio/cosmwasm/raw/0.7/lib/vm/testdata/contract_0.6.wasm",
            builder: "confio/cosmwasm-opt:0.6.2",
        },
    };
    const fee = {
        amount: [
            {
                amount: "5000000",
                denom: "ucosm",
            },
        ],
        gas: "89000000",
    };
    const { account_number, sequence } = (await client.authAccounts(testutils_spec_1.faucet.address)).result.value;
    const signBytes = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, account_number, sequence);
    const signature = await pen.sign(signBytes);
    const signedTx = makeSignedTx(theMsg, fee, memo, signature);
    return client.postTx(signedTx);
}
async function uploadContract(client, pen) {
    return uploadCustomContract(client, pen, testutils_spec_1.getHackatom());
}
async function instantiateContract(client, pen, codeId, beneficiaryAddress, transferAmount) {
    const memo = "Create an escrow instance";
    const theMsg = {
        type: "wasm/MsgInstantiateContract",
        value: {
            sender: testutils_spec_1.faucet.address,
            code_id: codeId.toString(),
            label: "my escrow",
            init_msg: {
                verifier: testutils_spec_1.faucet.address,
                beneficiary: beneficiaryAddress,
            },
            init_funds: transferAmount || [],
        },
    };
    const fee = {
        amount: [
            {
                amount: "5000000",
                denom: "ucosm",
            },
        ],
        gas: "89000000",
    };
    const { account_number, sequence } = (await client.authAccounts(testutils_spec_1.faucet.address)).result.value;
    const signBytes = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, account_number, sequence);
    const signature = await pen.sign(signBytes);
    const signedTx = makeSignedTx(theMsg, fee, memo, signature);
    return client.postTx(signedTx);
}
async function executeContract(client, pen, contractAddress) {
    const memo = "Time for action";
    const theMsg = {
        type: "wasm/MsgExecuteContract",
        value: {
            sender: testutils_spec_1.faucet.address,
            contract: contractAddress,
            msg: { release: {} },
            sent_funds: [],
        },
    };
    const fee = {
        amount: [
            {
                amount: "5000000",
                denom: "ucosm",
            },
        ],
        gas: "89000000",
    };
    const { account_number, sequence } = (await client.authAccounts(testutils_spec_1.faucet.address)).result.value;
    const signBytes = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, account_number, sequence);
    const signature = await pen.sign(signBytes);
    const signedTx = makeSignedTx(theMsg, fee, memo, signature);
    return client.postTx(signedTx);
}
describe("RestClient", () => {
    it("can be constructed", () => {
        const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
        expect(client).toBeTruthy();
    });
    // The /auth endpoints
    describe("authAccounts", () => {
        it("works for unused account without pubkey", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { height, result } = await client.authAccounts(testutils_spec_1.unused.address);
            expect(height).toMatch(testutils_spec_1.nonNegativeIntegerMatcher);
            expect(result).toEqual({
                type: "cosmos-sdk/Account",
                value: {
                    address: testutils_spec_1.unused.address,
                    public_key: "",
                    coins: [
                        {
                            amount: "1000000000",
                            denom: "ucosm",
                        },
                        {
                            amount: "1000000000",
                            denom: "ustake",
                        },
                    ],
                    account_number: testutils_spec_1.unused.accountNumber,
                    sequence: 0,
                },
            });
        });
        // This fails in the first test run if you forget to run `./scripts/wasmd/init.sh`
        it("has correct pubkey for faucet", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { result } = await client.authAccounts(testutils_spec_1.faucet.address);
            expect(result.value).toEqual(jasmine.objectContaining({
                public_key: pubkey_1.encodeBech32Pubkey(testutils_spec_1.faucet.pubkey, "cosmospub"),
            }));
        });
        // This property is used by CosmWasmClient.getAccount
        it("returns empty address for non-existent account", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const nonExistentAccount = testutils_spec_1.makeRandomAddress();
            const { result } = await client.authAccounts(nonExistentAccount);
            expect(result).toEqual({
                type: "cosmos-sdk/Account",
                value: jasmine.objectContaining({ address: "" }),
            });
        });
    });
    // The /blocks endpoints
    describe("blocksLatest", () => {
        it("works", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const response = await client.blocksLatest();
            // id
            expect(response.block_id.hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            // header
            expect(response.block.header.version).toEqual({ block: "10", app: "0" });
            expect(parseInt(response.block.header.height, 10)).toBeGreaterThanOrEqual(1);
            expect(response.block.header.chain_id).toEqual(testutils_spec_1.wasmd.chainId);
            expect(new readonly_date_1.ReadonlyDate(response.block.header.time).getTime()).toBeLessThan(readonly_date_1.ReadonlyDate.now());
            expect(new readonly_date_1.ReadonlyDate(response.block.header.time).getTime()).toBeGreaterThanOrEqual(readonly_date_1.ReadonlyDate.now() - 5000);
            expect(response.block.header.last_commit_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.last_block_id.hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.data_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.validators_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.next_validators_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.consensus_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.app_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.last_results_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.evidence_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.proposer_address).toMatch(testutils_spec_1.tendermintAddressMatcher);
            // data
            expect(response.block.data.txs === null || Array.isArray(response.block.data.txs)).toEqual(true);
        });
    });
    describe("blocks", () => {
        it("works for block by height", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const height = parseInt((await client.blocksLatest()).block.header.height, 10);
            const response = await client.blocks(height - 1);
            // id
            expect(response.block_id.hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            // header
            expect(response.block.header.version).toEqual({ block: "10", app: "0" });
            expect(response.block.header.height).toEqual(`${height - 1}`);
            expect(response.block.header.chain_id).toEqual(testutils_spec_1.wasmd.chainId);
            expect(new readonly_date_1.ReadonlyDate(response.block.header.time).getTime()).toBeLessThan(readonly_date_1.ReadonlyDate.now());
            expect(new readonly_date_1.ReadonlyDate(response.block.header.time).getTime()).toBeGreaterThanOrEqual(readonly_date_1.ReadonlyDate.now() - 5000);
            expect(response.block.header.last_commit_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.last_block_id.hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.data_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.validators_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.next_validators_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.consensus_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.app_hash).toMatch(testutils_spec_1.tendermintIdMatcher);
            expect(response.block.header.last_results_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.evidence_hash).toMatch(testutils_spec_1.tendermintOptionalIdMatcher);
            expect(response.block.header.proposer_address).toMatch(testutils_spec_1.tendermintAddressMatcher);
            // data
            expect(response.block.data.txs === null || Array.isArray(response.block.data.txs)).toEqual(true);
        });
    });
    // The /node_info endpoint
    describe("nodeInfo", () => {
        it("works", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { node_info, application_version } = await client.nodeInfo();
            expect(node_info).toEqual({
                protocol_version: { p2p: "7", block: "10", app: "0" },
                id: jasmine.stringMatching(testutils_spec_1.tendermintShortHashMatcher),
                listen_addr: "tcp://0.0.0.0:26656",
                network: testutils_spec_1.wasmd.chainId,
                version: jasmine.stringMatching(/^0\.33\.[0-9]+$/),
                channels: "4020212223303800",
                moniker: testutils_spec_1.wasmd.chainId,
                other: { tx_index: "on", rpc_address: "tcp://0.0.0.0:26657" },
            });
            expect(application_version).toEqual({
                name: "wasm",
                server_name: "wasmd",
                client_name: "wasmcli",
                version: jasmine.stringMatching(testutils_spec_1.semverMatcher),
                commit: jasmine.stringMatching(testutils_spec_1.tendermintShortHashMatcher),
                build_tags: "netgo,ledger",
                go: jasmine.stringMatching(/^go version go1\.[0-9]+\.[0-9]+ linux\/amd64$/),
            });
        });
    });
    // The /txs endpoints
    describe("txById", () => {
        let successful;
        let unsuccessful;
        beforeAll(async () => {
            if (testutils_spec_1.wasmdEnabled()) {
                const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
                const client = new signingcosmwasmclient_1.SigningCosmWasmClient(testutils_spec_1.wasmd.endpoint, testutils_spec_1.faucet.address, (signBytes) => pen.sign(signBytes));
                {
                    const recipient = testutils_spec_1.makeRandomAddress();
                    const transferAmount = {
                        denom: "ucosm",
                        amount: "1234567",
                    };
                    const result = await client.sendTokens(recipient, [transferAmount]);
                    successful = {
                        sender: testutils_spec_1.faucet.address,
                        recipient: recipient,
                        hash: result.transactionHash,
                    };
                }
                {
                    const memo = "Sending more than I can afford";
                    const recipient = testutils_spec_1.makeRandomAddress();
                    const transferAmount = [
                        {
                            denom: "ucosm",
                            amount: "123456700000000",
                        },
                    ];
                    const sendMsg = {
                        type: "cosmos-sdk/MsgSend",
                        value: {
                            // eslint-disable-next-line @typescript-eslint/camelcase
                            from_address: testutils_spec_1.faucet.address,
                            // eslint-disable-next-line @typescript-eslint/camelcase
                            to_address: recipient,
                            amount: transferAmount,
                        },
                    };
                    const fee = {
                        amount: [
                            {
                                denom: "ucosm",
                                amount: "2000",
                            },
                        ],
                        gas: "80000",
                    };
                    const { accountNumber, sequence } = await client.getNonce();
                    const chainId = await client.getChainId();
                    const signBytes = encoding_2.makeSignBytes([sendMsg], fee, chainId, memo, accountNumber, sequence);
                    const signature = await pen.sign(signBytes);
                    const signedTx = {
                        msg: [sendMsg],
                        fee: fee,
                        memo: memo,
                        signatures: [signature],
                    };
                    const transactionId = await client.getIdentifier({ type: "cosmos-sdk/StdTx", value: signedTx });
                    try {
                        await client.postTx(signedTx);
                    }
                    catch (error) {
                        // postTx() throws on execution failures, which is a questionable design. Ignore for now.
                        // console.log(error);
                    }
                    unsuccessful = {
                        sender: testutils_spec_1.faucet.address,
                        recipient: recipient,
                        hash: transactionId,
                    };
                }
                await utils_1.sleep(50); // wait until transactions are indexed
            }
        });
        it("works for successful transaction", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(successful);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txById(successful.hash);
            expect(result.height).toBeGreaterThanOrEqual(1);
            expect(result.txhash).toEqual(successful.hash);
            expect(result.codespace).toBeUndefined();
            expect(result.code).toBeUndefined();
            const logs = logs_1.parseLogs(result.logs);
            expect(logs).toEqual([
                {
                    msg_index: 0,
                    log: "",
                    events: [
                        {
                            type: "message",
                            attributes: [
                                { key: "action", value: "send" },
                                { key: "sender", value: successful.sender },
                                { key: "module", value: "bank" },
                            ],
                        },
                        {
                            type: "transfer",
                            attributes: [
                                { key: "recipient", value: successful.recipient },
                                { key: "sender", value: successful.sender },
                                { key: "amount", value: "1234567ucosm" },
                            ],
                        },
                    ],
                },
            ]);
        });
        it("works for unsuccessful transaction", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(unsuccessful);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txById(unsuccessful.hash);
            expect(result.height).toBeGreaterThanOrEqual(1);
            expect(result.txhash).toEqual(unsuccessful.hash);
            expect(result.codespace).toEqual("sdk");
            expect(result.code).toEqual(5);
            expect(result.logs).toBeUndefined();
            expect(result.raw_log).toContain("insufficient funds");
        });
    });
    describe("txsQuery", () => {
        let posted;
        beforeAll(async () => {
            if (testutils_spec_1.wasmdEnabled()) {
                const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
                const client = new signingcosmwasmclient_1.SigningCosmWasmClient(testutils_spec_1.wasmd.endpoint, testutils_spec_1.faucet.address, (signBytes) => pen.sign(signBytes));
                const recipient = testutils_spec_1.makeRandomAddress();
                const transferAmount = [
                    {
                        denom: "ucosm",
                        amount: "1234567",
                    },
                ];
                const result = await client.sendTokens(recipient, transferAmount);
                await utils_1.sleep(50); // wait until tx is indexed
                const txDetails = await new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint).txById(result.transactionHash);
                posted = {
                    sender: testutils_spec_1.faucet.address,
                    recipient: recipient,
                    hash: result.transactionHash,
                    height: Number.parseInt(txDetails.height, 10),
                    tx: txDetails,
                };
            }
        });
        it("can query transactions by height", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txsQuery(`tx.height=${posted.height}&limit=26`);
            expect(result).toEqual({
                count: "1",
                limit: "26",
                page_number: "1",
                page_total: "1",
                total_count: "1",
                txs: [posted.tx],
            });
        });
        it("can query transactions by ID", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txsQuery(`tx.hash=${posted.hash}&limit=26`);
            expect(result).toEqual({
                count: "1",
                limit: "26",
                page_number: "1",
                page_total: "1",
                total_count: "1",
                txs: [posted.tx],
            });
        });
        it("can query transactions by sender", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txsQuery(`message.sender=${posted.sender}&limit=200`);
            expect(parseInt(result.count, 10)).toBeGreaterThanOrEqual(1);
            expect(parseInt(result.limit, 10)).toEqual(200);
            expect(parseInt(result.page_number, 10)).toEqual(1);
            expect(parseInt(result.page_total, 10)).toEqual(1);
            expect(parseInt(result.total_count, 10)).toBeGreaterThanOrEqual(1);
            expect(result.txs.length).toBeGreaterThanOrEqual(1);
            expect(result.txs[result.txs.length - 1]).toEqual(posted.tx);
        });
        it("can query transactions by recipient", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txsQuery(`transfer.recipient=${posted.recipient}&limit=200`);
            expect(parseInt(result.count, 10)).toEqual(1);
            expect(parseInt(result.limit, 10)).toEqual(200);
            expect(parseInt(result.page_number, 10)).toEqual(1);
            expect(parseInt(result.page_total, 10)).toEqual(1);
            expect(parseInt(result.total_count, 10)).toEqual(1);
            expect(result.txs.length).toBeGreaterThanOrEqual(1);
            expect(result.txs[result.txs.length - 1]).toEqual(posted.tx);
        });
        it("can filter by tx.hash and tx.minheight", async () => {
            pending("This combination is broken 🤷‍♂️. Handle client-side at higher level.");
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const hashQuery = `tx.hash=${posted.hash}`;
            {
                const { count } = await client.txsQuery(`${hashQuery}&tx.minheight=0`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${hashQuery}&tx.minheight=${posted.height - 1}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${hashQuery}&tx.minheight=${posted.height}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${hashQuery}&tx.minheight=${posted.height + 1}`);
                expect(count).toEqual("0");
            }
        });
        it("can filter by recipient and tx.minheight", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const recipientQuery = `transfer.recipient=${posted.recipient}`;
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.minheight=0`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.minheight=${posted.height - 1}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.minheight=${posted.height}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.minheight=${posted.height + 1}`);
                expect(count).toEqual("0");
            }
        });
        it("can filter by recipient and tx.maxheight", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const recipientQuery = `transfer.recipient=${posted.recipient}`;
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.maxheight=9999999999999`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.maxheight=${posted.height + 1}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.maxheight=${posted.height}`);
                expect(count).toEqual("1");
            }
            {
                const { count } = await client.txsQuery(`${recipientQuery}&tx.maxheight=${posted.height - 1}`);
                expect(count).toEqual("0");
            }
        });
        it("can query by tags (module + code_id)", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const result = await client.txsQuery(`message.module=wasm&message.code_id=${testutils_spec_1.deployedErc20.codeId}`);
            expect(parseInt(result.count, 10)).toBeGreaterThanOrEqual(4);
            // Check first 4 results
            const [store, hash, isa, jade] = result.txs.map((tx) => testutils_spec_1.fromOneElementArray(tx.tx.value.msg));
            utils_1.assert(types_1.isMsgStoreCode(store));
            utils_1.assert(types_1.isMsgInstantiateContract(hash));
            utils_1.assert(types_1.isMsgInstantiateContract(isa));
            utils_1.assert(types_1.isMsgInstantiateContract(jade));
            expect(store.value).toEqual(jasmine.objectContaining({
                sender: testutils_spec_1.faucet.address,
                source: testutils_spec_1.deployedErc20.source,
                builder: testutils_spec_1.deployedErc20.builder,
            }));
            expect(hash.value).toEqual({
                code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                init_funds: [],
                init_msg: jasmine.objectContaining({
                    symbol: "HASH",
                }),
                label: "HASH",
                sender: testutils_spec_1.faucet.address,
            });
            expect(isa.value).toEqual({
                code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                init_funds: [],
                init_msg: jasmine.objectContaining({ symbol: "ISA" }),
                label: "ISA",
                sender: testutils_spec_1.faucet.address,
            });
            expect(jade.value).toEqual({
                code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                init_funds: [],
                init_msg: jasmine.objectContaining({ symbol: "JADE" }),
                label: "JADE",
                sender: testutils_spec_1.faucet.address,
            });
        });
        // Like previous test but filtered by message.action=store-code and message.action=instantiate
        it("can query by tags (module + code_id + action)", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            utils_1.assert(posted);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            {
                const uploads = await client.txsQuery(`message.module=wasm&message.code_id=${testutils_spec_1.deployedErc20.codeId}&message.action=store-code`);
                expect(parseInt(uploads.count, 10)).toEqual(1);
                const store = testutils_spec_1.fromOneElementArray(uploads.txs[0].tx.value.msg);
                utils_1.assert(types_1.isMsgStoreCode(store));
                expect(store.value).toEqual(jasmine.objectContaining({
                    sender: testutils_spec_1.faucet.address,
                    source: testutils_spec_1.deployedErc20.source,
                    builder: testutils_spec_1.deployedErc20.builder,
                }));
            }
            {
                const instantiations = await client.txsQuery(`message.module=wasm&message.code_id=${testutils_spec_1.deployedErc20.codeId}&message.action=instantiate`);
                expect(parseInt(instantiations.count, 10)).toBeGreaterThanOrEqual(3);
                const [hash, isa, jade] = instantiations.txs.map((tx) => testutils_spec_1.fromOneElementArray(tx.tx.value.msg));
                utils_1.assert(types_1.isMsgInstantiateContract(hash));
                utils_1.assert(types_1.isMsgInstantiateContract(isa));
                utils_1.assert(types_1.isMsgInstantiateContract(jade));
                expect(hash.value).toEqual({
                    code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                    init_funds: [],
                    init_msg: jasmine.objectContaining({
                        symbol: "HASH",
                    }),
                    label: "HASH",
                    sender: testutils_spec_1.faucet.address,
                });
                expect(isa.value).toEqual({
                    code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                    init_funds: [],
                    init_msg: jasmine.objectContaining({ symbol: "ISA" }),
                    label: "ISA",
                    sender: testutils_spec_1.faucet.address,
                });
                expect(jade.value).toEqual({
                    code_id: testutils_spec_1.deployedErc20.codeId.toString(),
                    init_funds: [],
                    init_msg: jasmine.objectContaining({ symbol: "JADE" }),
                    label: "JADE",
                    sender: testutils_spec_1.faucet.address,
                });
            }
        });
    });
    describe("encodeTx", () => {
        it("works for cosmoshub example", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            expect(await client.encodeTx(cosmoshub_json_1.default.tx)).toEqual(fromBase64(cosmoshub_json_1.default.tx_data));
        });
    });
    describe("postTx", () => {
        it("can send tokens", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
            const memo = "My first contract on chain";
            const theMsg = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: testutils_spec_1.faucet.address,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number, sequence } = (await client.authAccounts(testutils_spec_1.faucet.address)).result.value;
            const signBytes = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, account_number, sequence);
            const signature = await pen.sign(signBytes);
            const signedTx = makeSignedTx(theMsg, fee, memo, signature);
            const result = await client.postTx(signedTx);
            expect(result.code).toBeUndefined();
            expect(result).toEqual({
                height: jasmine.stringMatching(testutils_spec_1.nonNegativeIntegerMatcher),
                txhash: jasmine.stringMatching(testutils_spec_1.tendermintIdMatcher),
                // code is not set
                data: null,
                raw_log: jasmine.stringMatching(/^\[.+\]$/i),
                logs: jasmine.any(Array),
                gas_wanted: jasmine.stringMatching(testutils_spec_1.nonNegativeIntegerMatcher),
                gas_used: jasmine.stringMatching(testutils_spec_1.nonNegativeIntegerMatcher),
            });
        });
        it("can't send transaction with additional signatures", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const account1 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(0));
            const account2 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(1));
            const account3 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(2));
            const address1 = address_1.rawSecp256k1PubkeyToAddress(account1.pubkey, "cosmos");
            const address2 = address_1.rawSecp256k1PubkeyToAddress(account2.pubkey, "cosmos");
            const address3 = address_1.rawSecp256k1PubkeyToAddress(account3.pubkey, "cosmos");
            const memo = "My first contract on chain";
            const theMsg = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number: an1, sequence: sequence1 } = (await client.authAccounts(address1)).result.value;
            const { account_number: an2, sequence: sequence2 } = (await client.authAccounts(address2)).result.value;
            const { account_number: an3, sequence: sequence3 } = (await client.authAccounts(address3)).result.value;
            const signBytes1 = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, an1, sequence1);
            const signBytes2 = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, an2, sequence2);
            const signBytes3 = encoding_2.makeSignBytes([theMsg], fee, testutils_spec_1.wasmd.chainId, memo, an3, sequence3);
            const signature1 = await account1.sign(signBytes1);
            const signature2 = await account2.sign(signBytes2);
            const signature3 = await account3.sign(signBytes3);
            const signedTx = {
                msg: [theMsg],
                fee: fee,
                memo: memo,
                signatures: [signature1, signature2, signature3],
            };
            const postResult = await client.postTx(signedTx);
            // console.log(postResult.raw_log);
            expect(postResult.code).toEqual(4);
            expect(postResult.raw_log).toContain("wrong number of signers");
        });
        it("can send multiple messages with one signature", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const account1 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(0));
            const address1 = address_1.rawSecp256k1PubkeyToAddress(account1.pubkey, "cosmos");
            const memo = "My first contract on chain";
            const msg1 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const msg2 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "7654321",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number, sequence } = (await client.authAccounts(address1)).result.value;
            const signBytes = encoding_2.makeSignBytes([msg1, msg2], fee, testutils_spec_1.wasmd.chainId, memo, account_number, sequence);
            const signature1 = await account1.sign(signBytes);
            const signedTx = {
                msg: [msg1, msg2],
                fee: fee,
                memo: memo,
                signatures: [signature1],
            };
            const postResult = await client.postTx(signedTx);
            // console.log(postResult.raw_log);
            expect(postResult.code).toBeUndefined();
        });
        it("can send multiple messages with multiple signatures", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const account1 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(0));
            const account2 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(1));
            const address1 = address_1.rawSecp256k1PubkeyToAddress(account1.pubkey, "cosmos");
            const address2 = address_1.rawSecp256k1PubkeyToAddress(account2.pubkey, "cosmos");
            const memo = "My first contract on chain";
            const msg1 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const msg2 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address2,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "7654321",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number: an1, sequence: sequence1 } = (await client.authAccounts(address1)).result.value;
            const { account_number: an2, sequence: sequence2 } = (await client.authAccounts(address2)).result.value;
            const signBytes1 = encoding_2.makeSignBytes([msg2, msg1], fee, testutils_spec_1.wasmd.chainId, memo, an1, sequence1);
            const signBytes2 = encoding_2.makeSignBytes([msg2, msg1], fee, testutils_spec_1.wasmd.chainId, memo, an2, sequence2);
            const signature1 = await account1.sign(signBytes1);
            const signature2 = await account2.sign(signBytes2);
            const signedTx = {
                msg: [msg2, msg1],
                fee: fee,
                memo: memo,
                signatures: [signature2, signature1],
            };
            const postResult = await client.postTx(signedTx);
            // console.log(postResult.raw_log);
            expect(postResult.code).toBeUndefined();
            await utils_1.sleep(500);
            const searched = await client.txsQuery(`tx.hash=${postResult.txhash}`);
            expect(searched.txs.length).toEqual(1);
            expect(searched.txs[0].tx.value.signatures).toEqual([signature2, signature1]);
        });
        it("can't send transaction with wrong signature order (1)", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const account1 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(0));
            const account2 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(1));
            const address1 = address_1.rawSecp256k1PubkeyToAddress(account1.pubkey, "cosmos");
            const address2 = address_1.rawSecp256k1PubkeyToAddress(account2.pubkey, "cosmos");
            const memo = "My first contract on chain";
            const msg1 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const msg2 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address2,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "7654321",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number: an1, sequence: sequence1 } = (await client.authAccounts(address1)).result.value;
            const { account_number: an2, sequence: sequence2 } = (await client.authAccounts(address2)).result.value;
            const signBytes1 = encoding_2.makeSignBytes([msg1, msg2], fee, testutils_spec_1.wasmd.chainId, memo, an1, sequence1);
            const signBytes2 = encoding_2.makeSignBytes([msg1, msg2], fee, testutils_spec_1.wasmd.chainId, memo, an2, sequence2);
            const signature1 = await account1.sign(signBytes1);
            const signature2 = await account2.sign(signBytes2);
            const signedTx = {
                msg: [msg1, msg2],
                fee: fee,
                memo: memo,
                signatures: [signature2, signature1],
            };
            const postResult = await client.postTx(signedTx);
            // console.log(postResult.raw_log);
            expect(postResult.code).toEqual(8);
        });
        it("can't send transaction with wrong signature order (2)", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const account1 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(0));
            const account2 = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic, pen_1.makeSecretNetworkPath(1));
            const address1 = address_1.rawSecp256k1PubkeyToAddress(account1.pubkey, "cosmos");
            const address2 = address_1.rawSecp256k1PubkeyToAddress(account2.pubkey, "cosmos");
            const memo = "My first contract on chain";
            const msg1 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address1,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "1234567",
                        },
                    ],
                },
            };
            const msg2 = {
                type: "cosmos-sdk/MsgSend",
                value: {
                    from_address: address2,
                    to_address: emptyAddress,
                    amount: [
                        {
                            denom: "ucosm",
                            amount: "7654321",
                        },
                    ],
                },
            };
            const fee = {
                amount: [
                    {
                        amount: "5000",
                        denom: "ucosm",
                    },
                ],
                gas: "890000",
            };
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const { account_number: an1, sequence: sequence1 } = (await client.authAccounts(address1)).result.value;
            const { account_number: an2, sequence: sequence2 } = (await client.authAccounts(address2)).result.value;
            const signBytes1 = encoding_2.makeSignBytes([msg2, msg1], fee, testutils_spec_1.wasmd.chainId, memo, an1, sequence1);
            const signBytes2 = encoding_2.makeSignBytes([msg2, msg1], fee, testutils_spec_1.wasmd.chainId, memo, an2, sequence2);
            const signature1 = await account1.sign(signBytes1);
            const signature2 = await account2.sign(signBytes2);
            const signedTx = {
                msg: [msg2, msg1],
                fee: fee,
                memo: memo,
                signatures: [signature1, signature2],
            };
            const postResult = await client.postTx(signedTx);
            // console.log(postResult.raw_log);
            expect(postResult.code).toEqual(8);
        });
        it("can upload, instantiate and execute wasm", async () => {
            var _a;
            testutils_spec_1.pendingWithoutWasmd();
            const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const transferAmount = [
                {
                    amount: "1234",
                    denom: "ucosm",
                },
                {
                    amount: "321",
                    denom: "ustake",
                },
            ];
            const beneficiaryAddress = testutils_spec_1.makeRandomAddress();
            let codeId;
            // upload
            {
                // console.log("Raw log:", result.raw_log);
                const result = await uploadContract(client, pen);
                expect(result.code).toBeFalsy();
                const logs = logs_1.parseLogs(result.logs);
                const codeIdAttr = logs_1.findAttribute(logs, "message", "code_id");
                codeId = Number.parseInt(codeIdAttr.value, 10);
                expect(codeId).toBeGreaterThanOrEqual(1);
                expect(codeId).toBeLessThanOrEqual(200);
            }
            let contractAddress;
            // instantiate
            {
                const result = await instantiateContract(client, pen, codeId, beneficiaryAddress, transferAmount);
                expect(result.code).toBeFalsy();
                // console.log("Raw log:", result.raw_log);
                const logs = logs_1.parseLogs(result.logs);
                const contractAddressAttr = logs_1.findAttribute(logs, "message", "contract_address");
                contractAddress = contractAddressAttr.value;
                const amountAttr = logs_1.findAttribute(logs, "transfer", "amount");
                expect(amountAttr.value).toEqual("1234ucosm,321ustake");
                const balance = (await client.authAccounts(contractAddress)).result.value.coins;
                expect(balance).toEqual(transferAmount);
            }
            // execute
            {
                const result = await executeContract(client, pen, contractAddress);
                expect(result.code).toBeFalsy();
                // console.log("Raw log:", result.logs);
                const logs = logs_1.parseLogs(result.logs);
                const wasmEvent = (_a = logs.find(() => true)) === null || _a === void 0 ? void 0 : _a.events.find((e) => e.type === "wasm");
                utils_1.assert(wasmEvent, "Event of type wasm expected");
                expect(wasmEvent.attributes).toContain({ key: "action", value: "release" });
                expect(wasmEvent.attributes).toContain({
                    key: "destination",
                    value: beneficiaryAddress,
                });
                // Verify token transfer from contract to beneficiary
                const beneficiaryBalance = (await client.authAccounts(beneficiaryAddress)).result.value.coins;
                expect(beneficiaryBalance).toEqual(transferAmount);
                const contractBalance = (await client.authAccounts(contractAddress)).result.value.coins;
                expect(contractBalance).toEqual([]);
            }
        });
    });
    // The /wasm endpoints
    describe("query", () => {
        it("can list upload code", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            // check with contracts were here first to compare
            const existingInfos = await client.listCodeInfo();
            existingInfos.forEach((val, idx) => expect(val.id).toEqual(idx + 1));
            const numExisting = existingInfos.length;
            // upload data
            const wasmCode = testutils_spec_1.getHackatom();
            const result = await uploadCustomContract(client, pen, wasmCode);
            expect(result.code).toBeFalsy();
            const logs = logs_1.parseLogs(result.logs);
            const codeIdAttr = logs_1.findAttribute(logs, "message", "code_id");
            const codeId = Number.parseInt(codeIdAttr.value, 10);
            // ensure we were added to the end of the list
            const newInfos = await client.listCodeInfo();
            expect(newInfos.length).toEqual(numExisting + 1);
            const lastInfo = newInfos[newInfos.length - 1];
            expect(lastInfo.id).toEqual(codeId);
            expect(lastInfo.creator).toEqual(testutils_spec_1.faucet.address);
            // ensure metadata is present
            expect(lastInfo.source).toEqual("https://github.com/confio/cosmwasm/raw/0.7/lib/vm/testdata/contract_0.6.wasm");
            expect(lastInfo.builder).toEqual("confio/cosmwasm-opt:0.6.2");
            // check code hash matches expectation
            const wasmHash = new crypto_1.Sha256(wasmCode).digest();
            expect(lastInfo.data_hash.toLowerCase()).toEqual(toHex(wasmHash));
            // download code and check against auto-gen
            const { data } = await client.getCode(codeId);
            expect(fromBase64(data)).toEqual(wasmCode);
        });
        it("can list contracts and get info", async () => {
            testutils_spec_1.pendingWithoutWasmd();
            const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const beneficiaryAddress = testutils_spec_1.makeRandomAddress();
            const transferAmount = [
                {
                    amount: "707707",
                    denom: "ucosm",
                },
            ];
            // reuse an existing contract, or upload if needed
            let codeId;
            const existingInfos = await client.listCodeInfo();
            if (existingInfos.length > 0) {
                codeId = existingInfos[existingInfos.length - 1].id;
            }
            else {
                const uploadResult = await uploadContract(client, pen);
                expect(uploadResult.code).toBeFalsy();
                const uploadLogs = logs_1.parseLogs(uploadResult.logs);
                const codeIdAttr = logs_1.findAttribute(uploadLogs, "message", "code_id");
                codeId = Number.parseInt(codeIdAttr.value, 10);
            }
            // create new instance and compare before and after
            const existingContractsByCode = await client.listContractsByCodeId(codeId);
            for (const contract of existingContractsByCode) {
                expect(contract.address).toMatch(testutils_spec_1.bech32AddressMatcher);
                expect(contract.code_id).toEqual(codeId);
                expect(contract.creator).toMatch(testutils_spec_1.bech32AddressMatcher);
                expect(contract.label).toMatch(/^.+$/);
            }
            const result = await instantiateContract(client, pen, codeId, beneficiaryAddress, transferAmount);
            expect(result.code).toBeFalsy();
            const logs = logs_1.parseLogs(result.logs);
            const contractAddressAttr = logs_1.findAttribute(logs, "message", "contract_address");
            const myAddress = contractAddressAttr.value;
            const newContractsByCode = await client.listContractsByCodeId(codeId);
            expect(newContractsByCode.length).toEqual(existingContractsByCode.length + 1);
            const newContract = newContractsByCode[newContractsByCode.length - 1];
            expect(newContract).toEqual(jasmine.objectContaining({
                code_id: codeId,
                creator: testutils_spec_1.faucet.address,
                label: "my escrow",
            }));
            // check out info
            const myInfo = await client.getContractInfo(myAddress);
            utils_1.assert(myInfo);
            expect(myInfo.code_id).toEqual(codeId);
            expect(myInfo.creator).toEqual(testutils_spec_1.faucet.address);
            expect(myInfo.init_msg.beneficiary).toEqual(beneficiaryAddress);
            // make sure random addresses don't give useful info
            const nonExistentAddress = testutils_spec_1.makeRandomAddress();
            expect(await client.getContractInfo(nonExistentAddress)).toBeNull();
        });
        describe("contract state", () => {
            const client = new restclient_1.RestClient(testutils_spec_1.wasmd.endpoint);
            const noContract = testutils_spec_1.makeRandomAddress();
            const expectedKey = toAscii("config");
            let contractAddress;
            beforeAll(async () => {
                if (testutils_spec_1.wasmdEnabled()) {
                    const pen = await pen_1.Secp256k1Pen.fromMnemonic(testutils_spec_1.faucet.mnemonic);
                    const uploadResult = await uploadContract(client, pen);
                    utils_1.assert(!uploadResult.code);
                    const uploadLogs = logs_1.parseLogs(uploadResult.logs);
                    const codeId = Number.parseInt(logs_1.findAttribute(uploadLogs, "message", "code_id").value, 10);
                    const instantiateResult = await instantiateContract(client, pen, codeId, testutils_spec_1.makeRandomAddress());
                    utils_1.assert(!instantiateResult.code);
                    const instantiateLogs = logs_1.parseLogs(instantiateResult.logs);
                    const contractAddressAttr = logs_1.findAttribute(instantiateLogs, "message", "contract_address");
                    contractAddress = contractAddressAttr.value;
                }
            });
            it("can make smart queries", async () => {
                testutils_spec_1.pendingWithoutWasmd();
                // we can query the verifier properly
                const resultDocument = await client.queryContractSmart(contractAddress, { verifier: {} });
                expect(resultDocument).toEqual({ verifier: testutils_spec_1.faucet.address });
                // invalid query syntax throws an error
                await client.queryContractSmart(contractAddress, { nosuchkey: {} }).then(() => fail("shouldn't succeed"), (error) => expect(error).toMatch(/parsing hackatom::contract::QueryMsg: query contract failed/));
                // invalid address throws an error
                await client.queryContractSmart(noContract, { verifier: {} }).then(() => fail("shouldn't succeed"), (error) => expect(error).toMatch("not found"));
            });
        });
    });
});
//# sourceMappingURL=restclient.spec.js.map